--
-- HorschPronto6AS
-- Specialization for HorschPronto6AS
--
-- @author Felix "Outlaw" Sorge
-- @date  26/12/09
-- @web www.ls-modsource.de - www.ls-mods.de
--
-- Copyright (C) Outlaw, All Rights Reserved.
--


HorschPronto6AS = {};

Vehicle.registerJointType("horschImplement");

function HorschPronto6AS.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function HorschPronto6AS:load(xmlFile)
	
	self.setJointRotLimit = SpecializationUtil.callSpecializationsFunction("setJointRotLimit");
	
	self.arm2 = {};
	self.arm2.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.attacherJoint#index"));
	self.arm2.currentRotLimit = {0};
	
	
	self.seedPerSqm = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.seedPerSqm#value"), 1);
	self.horschAttacher = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.attacherJoint#index"));
	
	self.firstDo = {};
	self.arm = {};
	
end;


function HorschPronto6AS:update(dt)

	if self.attacherVehicle then
	
		if InputBinding.hasEvent(InputBinding.HorschPronto6AS_spurlinks) and self.attacherVehicle:getIsActiveForInput() then
			self.Speed.spurlinks = 2.8; 
			self.Go.spurlinks = not self.Go.spurlinks;
			self.Done.spurlinks = true; 
		end; 
		
		if InputBinding.hasEvent(InputBinding.HorschPronto6AS_spurchange) and self.attacherVehicle:getIsActiveForInput() and (self.Go.spurlinks or self.Go.spurrechts) then
			self.Speed.spurlinks = 2.8; 
			self.Speed.rechts = 2.8; 
			
			if self.Go.spurlinks then
				self.Go.spurlinks = false;
				self.Done.spurlinks = true; 
				self.Go.spurrechts = true;
				self.Done.spurrechts = true; 
			else
				self.Go.spurlinks = true;
				self.Done.spurlinks = true; 
				self.Go.spurrechts = false;
				self.Done.spurrechts = true; 
			end; 
		end; 
		
		if InputBinding.hasEvent(InputBinding.HorschPronto6AS_spurrechts) and self.attacherVehicle:getIsActiveForInput() then
			self.Speed.spurrechts = 2.8; 
			self.Go.spurrechts = not self.Go.spurrechts;
			self.Done.spurrechts = true; 
		end; 
	
		if InputBinding.hasEvent(InputBinding.HorschPronto6AS_trsp)  and self.attacherVehicle:getIsActiveForInput() and not self.attacherVehicle.Go.down and not self.attacherVehicle.CheckDone.down and not self.attacherVehicle.Done.down and not self.attacherVehicle.turnOn then
			self.Speed.trsp = 1.0; 
			self.Go.trsp = not self.Go.trsp;
			self.Done.trsp = true; 
			self.Speed.spurlinks = 2.8; 
			self.Speed.spurrechts = 2.8; 
			self.Go.spurlinks = false;
			self.Done.spurlinks = true; 
			self.Go.spurrechts = false;
			self.Done.spurrechts = true; 
		end; 
	
		if InputBinding.hasEvent(InputBinding.HorschPronto6AS_on)  and self.attacherVehicle:getIsActiveForInput() and self.CheckDone.trsp and self.Go.trsp then
			self.turnOn = not self.turnOn;
		end; 
		
		if InputBinding.hasEvent(InputBinding.HorschPronto6AS_markWheel)  and self.attacherVehicle:getIsActiveForInput() and self.CheckDone.trsp and self.Go.trsp then
			self.markWheel = not self.markWheel;
		end; 
					
		if self.attacherVehicle:getIsActiveForInput() and InputBinding.hasEvent(InputBinding.HorschPronto6AS_spurup) then
			self.Go.spurlinks = false;
			self.Done.spurlinks = true; 
			self.Go.spurrechts = false;
			self.Done.spurrechts = true; 
			self.Speed.spurlinks = 2.8; 
			self.Speed.spurrechts = 2.8; 
		end; 		
		
		
		if self.CheckDone.trsp then
			if self.attacherVehicle.Go.down and not self.attacherVehicle.Done.down and self.attacherVehicle.CheckDone.down  then
			
				if self.Go.spurlinks and not self.Done.spurlinks and self.CheckDone.spurlinks then
					local x,y,z = getWorldTranslation(self.cuttingAreas[4].start);
					local x1,y1,z1 = getWorldTranslation(self.cuttingAreas[4].width);
					local x2,y2,z2 = getWorldTranslation(self.cuttingAreas[4].height);
					local xg,yg,zg = getWorldTranslation(self.cuttingAreas[7].start);
					local x1g,y1g,z1g = getWorldTranslation(self.cuttingAreas[7].width);
					local x2g,y2g,z2g = getWorldTranslation(self.cuttingAreas[7].height);
					local cultivator = Utils.getDensity(g_currentMission.terrainDetailId, g_currentMission.cultivatorChannel, xg, zg, x1g, z1g, x2g, z2g);
					if cultivator ~= 0 then
						Utils.updatePloughArea(x, z, x1, z1, x2, z2);
					else
						Utils.updateCultivatorArea(x, z, x1, z1, x2, z2);	
					end; 
				end;
				
				if self.Go.spurrechts and not self.Done.spurrechts and self.CheckDone.spurrechts then
					local x,y,z = getWorldTranslation(self.cuttingAreas[5].start);
					local x1,y1,z1 = getWorldTranslation(self.cuttingAreas[5].width);
					local x2,y2,z2 = getWorldTranslation(self.cuttingAreas[5].height);
					local xg,yg,zg = getWorldTranslation(self.cuttingAreas[6].start);
					local x1g,y1g,z1g = getWorldTranslation(self.cuttingAreas[6].width);
					local x2g,y2g,z2g = getWorldTranslation(self.cuttingAreas[6].height);
					local cultivator = Utils.getDensity(g_currentMission.terrainDetailId, g_currentMission.cultivatorChannel, xg, zg, x1g, z1g, x2g, z2g);
					if cultivator ~= 0 then
						Utils.updatePloughArea(x, z, x1, z1, x2, z2);
					else
						Utils.updateCultivatorArea(x, z, x1, z1, x2, z2);	
					end; 
				end;
				
				if self.attacherVehicle.turnOn  and self.attacherVehicle.fillLevel ~= 0 then
					local x,y,z = getWorldTranslation(self.cuttingAreas[1].start);
					local x1,y1,z1 = getWorldTranslation(self.cuttingAreas[1].width);
					local x2,y2,z2 = getWorldTranslation(self.cuttingAreas[1].height);
					local area = Utils.updateSowingArea(self.attacherVehicle.currentFillType-3, x, z, x1, z1, x2, z2);
					local fruitDesc = FruitUtil.fruitIndexToDesc[self.attacherVehicle.currentFillType];
					local pixelToSqm = g_currentMission:getTerrainDetailPixelsToSqm(); -- 8192px are mapped to 2048m
					local sqm = area*pixelToSqm;
					local ha = sqm/10000;
					local usage = fruitDesc.seedUsagePerSqm*sqm;
					local difficultyMultiplier = math.max(3 * (3 - g_currentMission.missionStats.difficulty), 1);
					local deltaLevel = FruitUtil.fruitIndexToDesc[self.attacherVehicle.currentFillType].literPerSqm * difficultyMultiplier * usage * self.seedPerSqm;
					self.attacherVehicle:setFillLevel(self.attacherVehicle.fillLevel-deltaLevel, self.attacherVehicle.currentFillType);			
					g_currentMission.missionStats.seedUsageTotal = g_currentMission.missionStats.seedUsageTotal + usage;
					g_currentMission.missionStats.seedUsageSession = g_currentMission.missionStats.seedUsageSession + usage;
					g_currentMission.missionStats.hectaresSeededTotal = g_currentMission.missionStats.hectaresSeededTotal + ha;
					g_currentMission.missionStats.hectaresSeededSession = g_currentMission.missionStats.hectaresSeededSession + ha;
					local seedPrice = fruitDesc.seedPricePerLiter*usage;
					g_currentMission.missionStats.expensesTotal = g_currentMission.missionStats.expensesTotal + seedPrice;
					g_currentMission.missionStats.expensesSession = g_currentMission.missionStats.expensesSession + seedPrice;
					g_currentMission.missionStats.money = g_currentMission.missionStats.money - seedPrice;		 
					g_currentMission.missionStats.seedingDurationTotal = g_currentMission.missionStats.seedingDurationTotal + dt/(1000*60);
					g_currentMission.missionStats.seedingDurationSession = g_currentMission.missionStats.seedingDurationSession + dt/(1000*60);
				end; 	
				
				
				if self.markWheel then
					local x,y,z = getWorldTranslation(self.cuttingAreas[2].start);
					local x1,y1,z1 = getWorldTranslation(self.cuttingAreas[2].width);
					local x2,y2,z2 = getWorldTranslation(self.cuttingAreas[2].height);		
					Utils.updateCultivatorArea(x, z, x1, z1, x2, z2);					
					local x,y,z = getWorldTranslation(self.cuttingAreas[3].start);
					local x1,y1,z1 = getWorldTranslation(self.cuttingAreas[3].width);
					local x2,y2,z2 = getWorldTranslation(self.cuttingAreas[3].height);
					Utils.updateCultivatorArea(x, z, x1, z1, x2, z2);					
				end;

					
            end;
					
			
			x, y, z = getRotation(self.arm2.node);
			rot = {x,y,z};	
			local newRotLimit = {0,0,0};
			local rotDown = {30};
			local rotUp = {0};
			newRotLimit = Utils.getMovedLimitedValues(self.arm2.currentRotLimit, rotDown, rotUp, 1, 1000 * 2, dt, not self.attacherVehicle.Go.down);
			if math.abs(newRotLimit[1] - self.arm2.currentRotLimit[1]) > 0.001 then
				setJointRotationLimit(self.attacherVehicle.horschImplement.ObjectIndex, 0, true, Utils.degToRad(-newRotLimit[1]), Utils.degToRad(newRotLimit[1]));
			end;
			self.arm2.currentRotLimit = newRotLimit;	
			
			
			
			if not self.attacherVehicle.Go.down then
				self:onBrake(); 
			end;
		end;
					
						
					
		if self.Go.trsp then
			self.attacherVehicle.blockDetach = true; 
		elseif not self.Go.trsp and not self.Done.trsp and not self.CheckDone.trsp then
			self.attacherVehicle.blockDetach = false; 
		end; 
				
		if self.attacherVehicle.lightsActive then
			self:setLightsVisibility(true); 
		else
			self:setLightsVisibility(false); 
		end;
				
		if self.Go.spurlinks ~= nil and self.Done.spurlinks ~= false then
			self:anim("spurlinks", false); 
		end; 
		
		if self.Go.spurrechts ~= nil and self.Done.spurrechts ~= false then
			self:anim("spurrechts", false); 
		end; 
		
		
		if self.attacherVehicle.Go.down then

			self:setJointRotLimit(self.componentJoints[3],20, 0, 1000, not self.Go.spurlinks, 2,dt);	
			self:setJointRotLimit(self.componentJoints[5],20, 0, 1000, not self.Go.spurrechts, 2,dt);
				
		end; 	
		
		
		--draw start
		if self:getIsActive() then
		
			if self.markWheel and self.Go.trsp and self.CheckDone.trsp and not self.Done.trsp then
				g_currentMission:addHelpButtonText(g_i18n:getText("HorschPronto6AS_4"), InputBinding.HorschPronto6AS_markWheel); 
			elseif not self.markWheel and self.Go.trsp and self.CheckDone.trsp and not self.Done.trsp then
				g_currentMission:addHelpButtonText(g_i18n:getText("HorschPronto6AS_3"), InputBinding.HorschPronto6AS_markWheel); 
			end; 

	
			if self.Go.trsp and self.CheckDone.trsp and not self.Done.trsp then
		
				if self.Go.spurlinks then
					g_currentMission:addHelpButtonText(g_i18n:getText("HorschPronto6AS_6"), InputBinding.HorschPronto6AS_spurlinks); 
				else
					g_currentMission:addHelpButtonText(g_i18n:getText("HorschPronto6AS_5"), InputBinding.HorschPronto6AS_spurlinks); 
				end; 
			
			
				if self.Go.spurrechts then
					g_currentMission:addHelpButtonText(g_i18n:getText("HorschPronto6AS_8"), InputBinding.HorschPronto6AS_spurrechts); 
				else
					g_currentMission:addHelpButtonText(g_i18n:getText("HorschPronto6AS_7"), InputBinding.HorschPronto6AS_spurrechts); 
				end; 
			
				g_currentMission:addHelpButtonText(g_i18n:getText("HorschPronto6AS_9"), InputBinding.HorschPronto6AS_spurup); 
			
				if self.Go.spurrechts or self.Go.spurlinks then
					g_currentMission:addHelpButtonText(g_i18n:getText("HorschPronto6AS_10"), InputBinding.HorschPronto6AS_spurchange); 
				end; 
			end;
			
		end; 
		--draw end
		
	-- end active	
	end;
	
		
	local joint = self.componentJoints[3];
	setJointFrame(joint.jointIndex, 0,joint.jointNode);
	local joint = self.componentJoints[4];
	setJointFrame(joint.jointIndex, 0,joint.jointNode);
	local joint = self.componentJoints[5];
	setJointFrame(joint.jointIndex, 0,joint.jointNode);
	local joint = self.componentJoints[6];
	setJointFrame(joint.jointIndex, 0,joint.jointNode);
	
	
	if self.Done.trsp then
		local joint = self.componentJoints[1];
		setJointFrame(joint.jointIndex, 0,joint.jointNode);
		local joint = self.componentJoints[2];
		setJointFrame(joint.jointIndex, 0,joint.jointNode);
	end;	


	if self.Go.trsp ~= nil and self.Done.trsp ~= false then
		self:anim("trsp", false); 
	end; 
	

end;

function HorschPronto6AS:delete()
	if self.attacherVehicle then
		self.attacherVehicle.horschImplement = {}; 
	end; 
end;


function HorschPronto6AS:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicles then
		local trsp = getXMLBool(xmlFile, key.."#trsp");
		if trsp then
			self.Go.trsp = not self.Go.trsp; 
			self.Done.trsp = true; 
			self.Speed.trsp = 100.0;
		end; 
	end; 
    return BaseMission.VEHICLE_LOAD_OK;
end;

function HorschPronto6AS:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'trsp="'..tostring(self.CheckDone.trsp)..'"'; 
	return attributes, nil;
end;

function HorschPronto6AS:setJointRotLimit(nodei, up, down, speed, value, axle, dt)
	if not self.firstDo[nodei] then
		self.firstDo[nodei] = true;
		self.arm[nodei] = {0}; 
	end; 
	x, y, z = getRotation(nodei);
	rot = {x,y,z};	
	newRotLimit = {}; 
	newRotLimit[nodei] = {0,0,0};
	newRotLimit[nodei] = Utils.getMovedLimitedValues(self.arm[nodei], {down}, {up}, 1, speed * 2, dt, value);
	if math.abs(newRotLimit[nodei][1] - self.arm[nodei][1]) > 0.001 then
		local joint = nodei;
		setJointRotationLimit(joint.jointIndex, axle, true, Utils.degToRad(-newRotLimit[nodei][1]), Utils.degToRad(newRotLimit[nodei][1]));
	end;
	self.arm[nodei] = newRotLimit[nodei];			
end; 

function HorschPronto6AS:onDetach()
end;

function HorschPronto6AS:mouseEvent(posX, posY, isDown, isUp, button)
end;

function HorschPronto6AS:keyEvent(unicode, sym, modifier, isDown)
end;

function HorschPronto6AS:onAttach()
end; 	

function HorschPronto6AS:draw()		
end;